package com.gic.store.service.outer;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.dto.StoreExtendDTO;
import com.gic.store.dto.StoreTmpDTO;
import com.gic.store.entity.TabStoreBrand;
import com.gic.store.entity.TabStoreExtend;
import com.gic.store.entity.TabStoreGroup;
import com.gic.store.service.*;
import com.gic.store.utils.ErrorCode;
import com.task.allocation.exception.TaskAllocationException;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @date 2019/7/3
 */
@Service("storeImportApiService")
@SuppressWarnings("unchecked")
public class StoreImportApiServiceImpl implements StoreImportApiService {

    private static final Logger logger = LogManager.getLogger(StoreBrandApiServiceImpl.class);

    @Autowired
    private StoreBrandService storeBrandService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private StoreApiService storeApiService;

    @Autowired
    private StoreImportService storeImportService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private StoreTaskService storeTaskService;

    @Autowired
    private StoreExtendService storeExtendService;

    @Override
    public ServiceResponse<Boolean> hasImportingStore(Integer enterpriseId) {
        return ServiceResponse.success(storeImportService.hasImportingStore(enterpriseId));
    }

    @Override
    public void delStoreTmp(Integer enterpriseId) {
        storeImportService.delStoreTmp(enterpriseId);
    }

    @Override
    public void save(StoreTmpDTO bean) {
        storeImportService.save(bean);
    }

    @Override
    public ServiceResponse<Page> listStoreTmp(Integer enterpriseId, boolean isSuccess, Integer currentPage, Integer pageSize) {
        com.github.pagehelper.Page storeTmpList = storeImportService.listStoreTmp(enterpriseId, isSuccess, currentPage, pageSize);
        Page<StoreTmpDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeTmpList, StoreTmpDTO.class);
        List<StoreTmpDTO> result = page.getResult();
        for (StoreTmpDTO storeTmpDTO : result) {
            String customField = storeTmpDTO.getCustomField();
            List<StoreExtendDTO> extendList = storeExtendService.convertStoreExtendToDTO(JSON.parseArray(customField, TabStoreExtend.class));
            storeTmpDTO.setExtendList(extendList);
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Boolean> validateStoreCode(Integer enterpriseId, String storeCode) {
        return ServiceResponse.success(storeService.countByStoreCode(enterpriseId, storeCode, null) > 0);
    }

    @Override
    public ServiceResponse<String> isExistStoreGroup(Integer enterpriseId, String storeGroupName) {
        TabStoreGroup group = storeGroupService.getStoreGroupByName(storeGroupName, enterpriseId);
        return ServiceResponse.success(group == null ? null : group.getStoreGroupName());
    }

    @Override
    public ServiceResponse<List<String>> listBrandByNotExist(Integer enterpriseId, String brands) {
        if (StringUtils.isBlank(brands)) {
            return ServiceResponse.success();
        }
        List<String> resultList = new ArrayList<>();
        List<TabStoreBrand> storeBrandList = storeBrandService.listAllStoreBrand(enterpriseId, null, null, null);
        if (CollectionUtils.isEmpty(storeBrandList)) {
            return ServiceResponse.success();
        }
        List<String> storeBrandNameList = storeBrandList.stream().map(TabStoreBrand::getStoreBrandName).collect(Collectors.toList());
        String[] brandArr = brands.split("、");
        // 这里如果门店品牌很多， 可以改成map 去做
        for (String brandName : brandArr) {
            if (!storeBrandNameList.contains(brandName)) {
                resultList.add(brandName);
            }
        }

        return ServiceResponse.success(resultList);
    }

    @Override
    public ServiceResponse<List<StoreTmpDTO>> listStoreTmp(Integer enterpriseId, Boolean isRight) {
        List<StoreTmpDTO> result = EntityUtil.changeEntityListByOrika(StoreTmpDTO.class, storeImportService.listStoreTmp(enterpriseId, isRight));
        for (StoreTmpDTO storeTmpDTO : result) {
            String customField = storeTmpDTO.getCustomField();
            List<StoreExtendDTO> extendList = storeExtendService.convertStoreExtendToDTO(JSON.parseArray(customField, TabStoreExtend.class));
            storeTmpDTO.setExtendList(extendList);
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<List<StoreTmpDTO>> listAllStoreTmp(Integer enterpriseId) {
        return ServiceResponse.success(EntityUtil.changeEntityListByOrika(StoreTmpDTO.class, storeImportService.listAllStoreTmp(enterpriseId)));
    }

    @Override
    public ServiceResponse<List<List<String>>> listError(Integer enterpriseId) {
        List<List<String>> resultList = new LinkedList<>();

        List<StoreTmpDTO> dataList = this.listStoreTmp(enterpriseId, Boolean.FALSE).getResult();
        // title
        // 系统title
        // 自定义title
        List<String> title = getTitle(enterpriseId, dataList.get(0).getRegionId());
        resultList.add(title);
        // 数据
        // 普通数据
        // 自定义数据
        for (int i = 0; i < dataList.size(); i++) {
            List<String> list = new LinkedList<>();
            StoreTmpDTO storeTmpDTO = dataList.get(i);
            //{"序号", "门店名称", "门店代码", "门店类型", "门店状态", "门店电话", "门店所属分组", "关联门店品牌", "省", "市", "区县", "门店地址（不包括省市区）", "经度", "纬度"};
            list.add(i + 1 + "");
            list.add(storeTmpDTO.getStoreName());
            list.add(storeTmpDTO.getStoreCode());
            list.add(storeTmpDTO.getStoreType());
            list.add(storeTmpDTO.getErpStatus());
            list.add(storeTmpDTO.getConactsPhone());
            list.add(storeTmpDTO.getStoreGroupName());
            list.add(storeTmpDTO.getBrands());
            list.add(storeTmpDTO.getProvince());
            list.add(storeTmpDTO.getCity());
            list.add(storeTmpDTO.getCounty());
            list.add(storeTmpDTO.getAddress());
            list.add(storeTmpDTO.getLongitude());
            list.add(storeTmpDTO.getLatitude());

            List<StoreExtendDTO> extendList = storeTmpDTO.getExtendList();
            if (CollectionUtils.isNotEmpty(extendList)) {
                List<String> valueList = extendList.stream().map(StoreExtendDTO::getValue).collect(Collectors.toList());
                list.addAll(valueList);
            }
            list.add(storeTmpDTO.getErrorMessage());

            resultList.add(list);
        }
        return ServiceResponse.success(resultList);
    }

    private List<String> getTitle(Integer enterpriseId, Integer regionId) {
        List<String> titleList = storeApiService.listImportTitle(enterpriseId, regionId).getResult().get(0);
        titleList.add("错误信息");
        return titleList;
    }

    @Override
    public ServiceResponse<Page> pageError(Integer enterpriseId, Integer currentPage, Integer pageSize) {
        List<List<String>> resultList = new LinkedList<>();
        Page page = this.listStoreTmp(enterpriseId, Boolean.FALSE, currentPage, pageSize).getResult();
        List<StoreTmpDTO> result = page.getResult();


        return null;
    }


    @Override
    public ServiceResponse<String> importDataToStore(Integer enterpriseId, String userId, String uuId) {
        try {
            return ServiceResponse.success(storeTaskService.importDataToStore(enterpriseId, userId, uuId));
        } catch (TaskAllocationException e) {
            logger.warn("队列错误");
        }
        return ServiceResponse.failure(ErrorCode.ERR_3.getCode(), "队列错误");
    }

}
